rm(list=ls())
###############################################################
####
####  Copy Number recognition in  Illuymina arrays
####
####   This procedure takes segm_Sample1 files, applies GAP parameters from GAP_parameters_file,
####     and makes estimation of segmental Copy number and Genotype.
####    Results are saved in Results_Plots folder as: 
####      (1) proc_Sample1 text file of all segments and copy numbers
####      (2) Chr_Plot_Sample1 per chromosome profiles .jpg
####      (3) Summary_Plot_Sample1 summary profiles and information
####      (4) GAP_Plot_Sample1 plot of GAP and copy number detection
####
##################################################################


### !!!!! Put the PATH to your data  !!!!

pPath<-"C:/Users/Tatiana/Desktop/GAP_Illumina_scripts"


inputPath<-paste(pPath,"Vectors_and_segmentations",sep="/")
outputPath<-paste(pPath,"Results_Plots",sep="/")


#### Read Annotation file

SNP_annot_fileName<-"Illum660K_annot_cut.csv"


SNP_annot<-read.csv(paste(pPath,SNP_annot_fileName,sep="/"),header=T,fill=T,stringsAsFactors = F,row.names=1)
SNP_annot<-data.matrix(SNP_annot)
C_Ind<-1;C_Chr<-2;C_Pos<-3

nSNP<-dim(SNP_annot)[[1]]


### Source file needs SNP_annot file defined above
source(paste(pPath,"script_functions_GAP_recognition_Illumina.R",sep="/"))

### !!!!! Give your favorit name to output_infoFileName, if you want   !!!!
output_infoFileName<-"GAPrecognitionInfo.txt"


######## Structure of output summary information file concerning recognition
######## and rearrangements for all samples in the list

info<-c("p_BAF","q_LRR","2copyLRR","Breaks","DNAindex","Ploidy","Chr_counts")
cat(c("SampleID",info), file = paste(outputPath,output_infoFileName,sep="/"),sep="\t","\n")


##################################################################
####
####  !!!! Cut-offs !!!! Correct, if necessary !!!!
####
####  germHomozyg.mBAF.thr is the most important cut-off for recognition
####  If using Affymetrix 100,250,500, etc, germHomozyg.mBAF.thr need to be around  0.85!!!!
####  This cut-off is used to eliminate homozygous SNPs
####     
####  germHet.Proportion is minimal proportion of heterozygous SNPs in the segment to be called Het
####  If the segment has less than germHet.Proportion heterozyg SNPs it is called Homozygous

germHomozyg.mBAF.thr<-0.93
germHet.Proportion<-(1/10)


### !!!!! All segm_Sample1 files from Vectors_and_Segmentations folder will be processed !!!!

segFiles <- list.files(inputPath, pattern=vect_Segm_pattern,full.names=T) 

fileNames <- gsub(inputPath,"",segFiles) 
fileNames <- gsub(vect_Segm_pattern,"",fileNames) 
fileNames <- gsub(".txt","",fileNames) 
fileNames <- gsub("/","",fileNames) 

fileNames

nSample<-4
nSample<-length(segFiles)

### !!!!! Check the GAP_parameters_file_name, te be the same as after script_3 or set chooseGAPparameterFromFile to FALSE  !!!!

#chooseGAPparameterFromFile<-TRUE
chooseGAPparameterFromFile<-FALSE

GAP_parameters_file_name<-"GAP_parameters_Validation_LCC.csv"

if(chooseGAPparameterFromFile){
if(file.access(paste(inputPath,GAP_parameters_file_name,sep="/"), mode = 0)==0){
	GAP_par_exist<-T
	GAP_parameters_file<-read.csv(paste(inputPath,GAP_parameters_file_name,sep="/"),stringsAsFactors = F,header=T,row.names=1)
}else{GAP_par_exist<-F}
}


####################   MAIN CYCLE   #####################################

ii<-1


for(ii in 1:nSample){

print(paste(as.character(ii),"out of",as.character(nSample),"processing",fileNames[ii],sep="  "))

#################################################################
####
#### Step 1.  Determination of GAP pattern
####  		functions: CopyNumber_and_Genotype_pattern
####               and  CopyNumber_and_Genotype_patternWOfitting
####
##################################################################

#tmp<-read.delim(segFiles[ii],header=T,sep=" ")

germHomozyg.mBAF.thr<-0.93
germHet.Proportion<-(1/10)


	results<-readSegmFile(segFileName=segFiles[ii])
	tmp<-results$tmp
	p_BAF<-results$p_BAF
	q_LRR<-results$q_LRR
	Delta<-results$Delta	
	germHomThr<-results$germHomThr
	germProp<-results$germProp

if(!is.na(germHomThr)){germHomozyg.mBAF.thr<-germHomThr}
if(!is.na(germProp)){germHet.Proportion<-germProp}

notNAs<-sort(intersect(which(!is.na(tmp[,c_lrr]+tmp[,c_baf])),which(tmp[,c_chr]<23)))


plot(tmp[notNAs,c_baf],tmp[notNAs,c_lrr],cex=tmp[notNAs,c_len]/quantile(tmp[notNAs,c_len],prob=0.9),ylim=c(max(-2,min(tmp[notNAs,c_lrr])),min(2,max(tmp[notNAs,c_lrr]))),
		main="GAP",xlab="B allele frequency",ylab="Log R ratio")
abline(v=0)
abline(v=0.5)


############### Comments to notNAs
##### NAs are not allowed in recognition function
##### Sex chromosomes measurements on Illumina SNP arrays are usually shifted 
##### Y chromosome in female sample gives oulying values, so
##### it is a strong reason to exclude sex chromosomes from pattern recognition procedure


###################################################################
###
### Three ways of dealing with GAP pattern recognition function: 
###
### 1. If GAP parameters were precalculated, use function CopyNumber_and_Genotype_patternWOfitting
###  The function produce all necessary things for recognition with the given constants
###
###  2. If GAP parameters were precalculated AND saved in the first line of segm_Sample
###
###  3. If performing recognition de novo use function CopyNumber_and_Genotype_pattern

###  4. Use this function also if automatical recognition fails to fit the model correctly, while
###   one can see the right solution by visual inspection of the GAP pattern. 
###   Introduce corresponding values in the model and look at resulting pattern
###   Adjust parameters if necessary



##### Choose parameters from GAP_parameters_file

GAP_parameters<-F

if(chooseGAPparameterFromFile){
	
	if(GAP_par_exist){
		ii_name<-grep(fileNames[ii],rownames(GAP_parameters_file))
		if(length(ii_name)!=1){
			if(length(ii_name)>1){
				ii_name1<-numeric(0)
				for(i in 1:length(ii_name)){
					if(length(grep(rownames(GAP_parameters_file)[ii_name[i]],fileNames[ii]))==1){ii_name1<-c(ii_name1,i)}
				}
				if(length(ii_name1)==1){ii_name<-ii_name[ii_name1];GAP_parameters<-T}
			}
		}else{(length(grep(rownames(GAP_parameters_file)[ii_name],fileNames[ii]))==1){GAP_parameters<-T}}
}}


if(GAP_parameters){

### If precalculated GAP parameters: case 1.

p_BAF<-GAP_parameters_file[ii_name,2];q_LRR<-GAP_parameters_file[ii_name,3];Delta<-c(2,GAP_parameters_file[ii_name,4])
result<-CopyNumber_and_GenotypeWOfitting(tmp[notNAs,c_lrr],tmp[notNAs,c_baf],tmp[notNAs,c_len],showGAPTemplate=T,p_BAF,q_LRR,Delta)

}else{

### If taking GAP parameters from segm file: case 2

tt<-which(is.na(c(p_BAF,q_LRR,Delta[2])))
if(length(tt)==0){

result<-CopyNumber_and_GenotypeWOfitting(tmp[notNAs,c_lrr],tmp[notNAs,c_baf],tmp[notNAs,c_len],showGAPTemplate=T,p_BAF,q_LRR,Delta)

}else{

### If calculating GAP parameters de novo: case 3.

result<-CopyNumber_and_Genotype(tmp[notNAs,c_lrr],tmp[notNAs,c_baf],tmp[notNAs,c_len],showGAPTemplate=T)

}
}

#### My parameters: case 4.
#### Delta is the vector of 2: c(copy number, LR value)
#### For example, sample BLC_T31 automatically detected as diploid, but, it looks like just dublicated diploid (near-tetraploid)
#### so, we introduce:p_BAF<-0.2;q_LRR<-0.3;Delta<-c(4,0.07)
#### result<-CopyNumber_and_GenotypeWOfitting(tmp[notNAs,c_lrr],tmp[notNAs,c_baf],tmp[notNAs,c_len],showGAPTemplate=T,p_BAF,q_LRR,Delta)

### coefficients of the pattern shrinkage
p_BAF<-result$coeff[1]
q_LRR<-result$coeff[2]

### matrix of coordinates of claster centers
Centr_CL<-result$Centr_CL

### matrix of clusters on the LRR x BAF plane
NETM<-result$NETM

### matrix of coordinates of subclones
SB_CL<-result$SB_CL

rm(result)

############# GAP pattern recognition is completed

###################### Comments to GAP recognition results
#### As a result we have got: 
####		1) p_BAF and q_LRR: parameters of the model GAP
####		2) Centr_CL:  a table with coordinates of model GAP clusters
####		3) NETM: a matrix enumerating all grid nodes and their cluster annotation 
####			(Index in columname of NETM = Index in rownames of Centr_CL)
####		4) SB_CL: a matrix enumerating all segments positioned essentially inbetween clusters
####			(Index_Subclone in rowname of SB_CL = Index in columname of NETM)
#### These data are used for copy number and genotype attribution to each segment in tmp table


##########################################################
############## Output segmented summary file format with Copy Number, Major allele and other supplemetaries

tmp<-data.matrix(tmp[,1:10])

tmp<-cbind(tmp,matrix(0,dim(tmp)[[1]],16))
colnames(tmp)[11:26]<-ColNamesTMP[11:26]

recCHR<-getRecCHR()

tmp[,c_nC]<-getCNwithPrecision(tmp[,c_lrr],Centr_CL,Precision=1/8)



tmp<-getGTwithRank(tmp,Centr_CL,NETM)


tmp[,c_CN]<-tmp[,c_CN1]
tmp[,c_BA]<-tmp[,c_BA1]

######################### Comments to CN recognition
### function getCNwithPrecision determines the closest Copy number from the GAP pattern to each smoothed Log Ratio value
### Segments within Precision relative distance around CN level are attributed pure CN status
### Segments beyond Precision relative distance around CN level are attributed rational CN 
### (such as 2.3, meaning at 3/10 to 2 copy and 7/10 to 3 copy level)
### This is just subsidiary values to check BAF in the function: getGTwithRank
### (segments with 2.3 CN status are checked for the closest cluster in 2 and 3 copies levels)
### (segments with 2 CN status are checked to the closest cluster only in 2 copy level)



##################### Comments (1) to recognition results
##### If segmentation was good (with not so many redundant break points), if all profiles are
#####  not very noisy, then the results of recognition are ready.
#####    CN1,BA1 are taken as recognized copy number and B allele counts.
#####  GoTo the output section.
#####
#####   (0 in B alelle means germline homozygosity with undeterminable somatic allelic status)
#####


########
############## If segmentation is redundant and many false breaks and attributions, we try to smooth the profile
########  LengthMinRec is the length of a segment which we consider to be "short", could be changed

LengthMinRec<-trunc(dim(SNP_annot)[[1]]/60000+1)*10

tmp<-fitShortSubclones(tmp,Centr_CL,NETM,p_BAF,Length=LengthMinRec)

tmp<-shortBreaksToTMP(tmp,CHR=recCHR)

####### Comments to fitShortSubclones and shortBreaksToTMP
####### These functions are very effective, however, not well tested and debugged
####### If many breakpoints and all the segments within chromosome are short (<50 SNP) 
#######  they could fall into infinite cycle and produce unstable profiles.


###############################################
####### Save table with possible short losses (might be important for cancer genome study)


##### For this procedure the vector of LRR and BAF is necessary
datA<-read.table(sub(vect_Segm_pattern,vect_BAF_pattern,segFiles[ii]),header=F,as.is=T)
datA<-data.matrix(datA)
datA<-as.numeric(datA)
datA<-abs(datA-0.5)+0.5

datL<-read.table(sub(vect_Segm_pattern,vect_LR_pattern,segFiles[ii]),header=F,as.is=T)
datL<-data.matrix(datL)
datL<-as.numeric(datL)



TMP<-tmp
loss<-which(TMP[,c_nC]<0.4)
if(length(loss)>0){
	tt<-which(TMP[loss,c_len]>=15)
	if(length(tt)>0){loss<-loss[tt]
		tt<-which(is.na(TMP[loss,c_lrr]))
		if(length(tt)>0){loss<-loss[-tt]}
		if(length(loss)>0){TMP[loss,c_conf]<-0}
}}

TMP<-iliminateShort_FinalPolish(TMP,Short=20)
TMP<-getConfidence(TMP)

LengthHomoMinRec<-trunc(dim(SNP_annot)[[1]]/20000+1)*10

TMP<-checkSmallHomozygosities_FinalPolish(TMP,Length=LengthHomoMinRec)

TMP<-checkHomozygosities_FinalPolish(TMP,Length=LengthHomoMinRec+1,p_BAF)
TMP<-checkHomozygBreaks_finalPolish(TMP)
TMP<-checkSmallDiscrepancies_finalPolish(TMP,Length=trunc(LengthHomoMinRec/2))
TMP<-checkSmallHomozygosities_FinalPolish(TMP,Length=LengthHomoMinRec)

TMP<-getConfidence(TMP)

TMP<-updateMeans_FinalPolish(TMP,datA,datL,germHomoThr=germHomozyg.mBAF.thr)

TMP<-cbind(TMP[,1:3],TMP[,2:dim(TMP)[[2]]])
TMP[,4]<-SNP_annot[TMP[,c_is],C_Pos]
TMP[,5]<-SNP_annot[TMP[,c_if],C_Pos]
colnames(TMP)[4:5]<-c("PosSNPstart","PosSNPend")


	info<-as.character(round(c(p_BAF,q_LRR,Centr_CL[5,3],germHomThr,germProp),2))
	outputInfo<-paste("##","Annotations=",SNP_annot_fileName, "; p_BAF=",info[1],"; q_LRR=",info[2],";2LRR=",info[3], ";germHomozyg_BAF_thr=",info[4],";germHet_Proportion=",info[5],sep="")

fName<-paste(outputPath,paste("proc_",fileNames[ii],".txt",sep=""),sep="/")
write.table(outputInfo, file = fName,quote=F,row.names=F,col.names=F)
write.table(TMP, file = fName,quote=F,row.names=F,sep="\t",append=T)

##############################################
####### Summary figures

TMP<-TMP[,-c(4,5)]
TMP<-shrinkReprTMP(TMP)
TMP<-updateMeans_FinalPolish(TMP,datA,datL,germHomoThr=germHomozyg.mBAF.thr)
summaryPlots_perChrom(TMP,nameSample=fileNames[ii],SNP_annot, datA, datL, Centr_CL)

TMP<-iliminateShort_FinalPolish(TMP,Short=50)
TMP<-shrinkReprTMP(TMP)
TMP<-updateMeans_FinalPolish(TMP,datA,datL,germHomoThr=germHomozyg.mBAF.thr)

GAPplot(TMP,nameSample=fileNames[ii],SNP_annot,Centr_CL)

info<-summaryPlots(TMP,nameSample=fileNames[ii], SNP_annot, datA, datL, Centr_CL)

cat(c(fileNames[ii],info), file = paste(outputPath,output_infoFileName,sep="/"),sep="\t",append=T,"\n")

}##ii

###################### END MAIN CYCLE   ########################################

#dev.off()
